from django.conf import settings
from django.contrib.auth import views as auth_views
from django.contrib.auth.decorators import user_passes_test
from django.urls import include, path, reverse
from django.utils.functional import lazy

from . import views

login_forbidden = user_passes_test(
    lambda u: u.is_anonymous, lazy(reverse, str)("dashboard"), redirect_field_name=None
)

urlpatterns = [
    path("", views.dashboard, name="dashboard"),
    path("login/", login_forbidden(views.login), name="login"),
    path(
        "signup/",
        login_forbidden(views.signup),
        name="signup",
    ),
    path("forgot-password/", views.PasswordResetView.as_view(), name="password_reset"),
    path(
        "forgot-password/done/",
        views.PasswordResetDoneView.as_view(),
        name="password_reset_done",
    ),
    path(
        "reset/<str:uidb64>/<str:token>",
        views.PasswordResetConfirmView.as_view(),
        name="password_reset_confirm",
    ),
    path("sign_out/", auth_views.LogoutView.as_view(), name="sign_out"),
    path("account/", views.account, name="account"),
    path("change-password/", views.change_password, name="change_password"),
]

if settings.DEBUG:
    import debug_toolbar

    urlpatterns += [path("__debug__/", include(debug_toolbar.urls))]



from crispy_forms.helper import FormHelper
from crispy_forms.layout import BaseInput
from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import UserChangeForm, UserCreationForm
from django.forms import ModelForm


class CustomUserCreationForm(UserCreationForm):
    class Meta(UserCreationForm):
        model = get_user_model()
        fields = ("email",)


class CustomUserChangeForm(UserChangeForm):
    class Meta:
        model = get_user_model()
        fields = ("email",)


class Button(BaseInput):
    input_type = "submit"

    def __init__(self, *args, **kwargs):
        self.field_classes = (
            "inline-flex justify-center py-3 px-6 border border-transparent shadow-sm text-base font-medium"
            + "rounded-md text-white bg-pc-600 hover:bg-pc-700 focus:outline-none focus:ring-2"
            + "focus:ring-offset-2 focus:ring-pc-500"
        )
        super().__init__(*args, **kwargs)


class EmptyChoiceField(forms.ChoiceField):
    def __init__(
        self,
        choices=(),
        empty_label=None,
        required=True,
        widget=None,
        label=None,
        initial=None,
        help_text=None,
        *args,
        **kwargs
    ):

        # prepend an empty label if it exists (and field is not required!)
        if not required and empty_label is not None:
            choices = tuple([("", empty_label)] + list(choices))

        super().__init__(
            choices=choices,
            required=required,
            widget=widget,
            label=label,
            initial=initial,
            help_text=help_text,
            *args,
            **kwargs
        )


class ButtonFullWidth(BaseInput):
    input_type = "submit"

    def __init__(self, *args, **kwargs):
        self.field_classes = (
            "w-full flex justify-center py-2 px-4 border border-transparent rounded-md shadow-sm text-sm"
            + "font-medium text-white bg-pc-600 hover:bg-pc-700 focus:outline-none focus:ring-2"
            + "focus:ring-offset-2 focus:ring-pc-500"
        )
        super().__init__(*args, **kwargs)


class TailwindFormMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_class = "grid grid-cols-1 gap-y-6"
        self.helper.label_class = "block text-sm font-medium text-gray-700 mb-1"


class TailwindForm(TailwindFormMixin, forms.Form):
    pass


class TailwindModelForm(TailwindFormMixin, ModelForm):
    pass



import datetime

from django import template
from django.template import RequestContext, defaultfilters, loader
from django.urls import reverse

from tuhls.dashboard import menu

register = template.Library()


@register.simple_tag(takes_context=True)
def render_menu(context):
    return loader.get_template("dashboard/menu.html").render(
        {**context.flatten(), **{"menu": menu}}
    )


@register.simple_tag(takes_context=True)
def render_field(context: RequestContext, instance, field):
    def get_value():
        if field in context:
            return context[field]
        if "view" in context and hasattr(context["view"], field):
            return getattr(context["view"], "preview")(instance)
        if hasattr(instance, field):
            return getattr(instance, field)

    def autoformat(value):
        if isinstance(value, str):
            return value
        if isinstance(value, datetime.datetime):
            return defaultfilters.date(value) + " " + defaultfilters.time(value)
        if isinstance(value, datetime.date):
            return defaultfilters.date(value)
        if isinstance(value, datetime.time):
            return defaultfilters.time(value)

    return autoformat(get_value())


@register.simple_tag(takes_context=True)
def dashboard_url(context: RequestContext, action_name, *args, **kwargs):
    return reverse(
        context["request"].resolver_match.app_name + ":" + action_name,
        None,
        kwargs=kwargs,
    )
